@@ -2,7 +2,7 @@
     YoshimiLV2Plugin
 
     Copyright 2014, Andrew Deryabin <andrewderyabin@gmail.com>
-    Copyright 2016-2021, Will Godfrey & others.
+    Copyright 2016-2024, Will Godfrey, Kristian Amlie, Ichthyostega and others.
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -32,6 +32,12 @@
     #include "MasterUI.h"
 #endif
 
+#include <memory>
+#include <string>
+#include <thread>
+
+using std::string;
+using std::make_unique;
 
 #define YOSHIMI_STATE_URI "http://yoshimi.sourceforge.net/lv2_plugin#state"
 
@@ -50,7 +56,6 @@
 
 #define YOSHIMI_LV2_STATE__StateChanged      "http://lv2plug.in/ns/ext/state#StateChanged"
 
-extern SynthEngine *firstSynth;
 
 
 typedef enum {
@@ -62,12 +67,12 @@ typedef enum {
 
 
 typedef struct _Yoshimi_LV2_Options_Option {
- Yoshimi_LV2_Options_Context context;  /**< Context (type of subject). */
- uint32_t            subject;  /**< Subject. */
- LV2_URID            key;      /**< Key (property). */
- uint32_t            size;     /**< Size of value in bytes. */
- LV2_URID            type;     /**< Type of value (datatype). */
- const void*         value;    /**< Pointer to value (object). */
+ Yoshimi_LV2_Options_Context context;  ///< Context (type of subject).
+                    uint32_t subject;  ///< Subject.
+                    LV2_URID key;      ///< Key (property).
+                    uint32_t size;     ///< Size of value in bytes.
+                    LV2_URID type;     ///< Type of value (datatype).
+                 const void* value;    ///< Pointer to value (object).
 } Yoshimi_LV2_Options_Option;
 
 
@@ -98,13 +103,17 @@ LV2_Descriptor yoshimi_lv2_multi_desc =
     YoshimiLV2Plugin::extension_data
 };
 
+namespace {
+    inline bool isMultiFeed(LV2_Descriptor const& desc)
+    {
+        return string{desc.URI} == string{yoshimi_lv2_multi_desc.URI};
+    }
+}
+
 
 void YoshimiLV2Plugin::process(uint32_t sample_count)
 {
-    if (sample_count == 0)
-    {
-        return;
-    }
+    if (sample_count == 0) return;  // explicitly allowed by LV2 standard
 
     /*
      * Our implementation of LV2 has a problem with envelopes. In general
@@ -114,15 +123,13 @@ void YoshimiLV2Plugin::process(uint32_t sample_count)
      * However, Yoshimi is always correct when working standalone.
      */
 
-    int offs = 0;
-    uint32_t next_frame = 0;
     uint32_t processed = 0;
     BeatTracker::BeatValues beats(beatTracker->getRawBeatValues());
     uint32_t beatsAt = 0;
     bool bpmProvided = false;
     float *tmpLeft [NUM_MIDI_PARTS + 1];
     float *tmpRight [NUM_MIDI_PARTS + 1];
-    struct midi_event intMidiEvent;
+    MidiEvent intMidiEvent;
     for (uint32_t i = 0; i < NUM_MIDI_PARTS + 1; ++i)
     {
         tmpLeft [i] = lv2Left [i];
@@ -137,41 +144,43 @@ void YoshimiLV2Plugin::process(uint32_t sample_count)
         if (event == NULL)
             continue;
 
+        uint32_t next_frame = event->time.frames;
+        if (next_frame >= sample_count)
+            continue;
+
+        // Avoid sample perfect alignment when not free wheeling (running
+        // offline, as when rendering a track), because it is extremely
+        // expensive when there are many MIDI events with just small timing
+        // differences. It is also not real time safe, because the amount of
+        // processing depends on the timing of the notes, not only by the number
+        // of notes. Let the user control the granularity using buffer size
+        // instead.
+        uint32_t frameAlignment;
+        if (isFreeWheel())
+            frameAlignment = 1;
+        else
+            frameAlignment = synth.buffersize;
+        while (next_frame - processed >= frameAlignment)
+        {
+            float bpmInc = (float)(processed - beatsAt) * beats.bpm / (synth.samplerate_f * 60.f);
+            synth.setBeatValues(beats.songBeat + bpmInc, beats.monotonicBeat + bpmInc, beats.bpm);
+            int mastered_chunk = synth.MasterAudio(tmpLeft, tmpRight, next_frame - processed);
+            for (uint32_t i = 0; i < NUM_MIDI_PARTS + 1; ++i)
+            {
+                tmpLeft [i] += mastered_chunk;
+                tmpRight [i] += mastered_chunk;
+            }
+            processed += mastered_chunk;
+        }
+
         if (event->body.type == _midi_event_id)
         {
             if (event->body.size > sizeof(intMidiEvent.data))
                 continue;
 
-            next_frame = event->time.frames;
-            if (next_frame >= sample_count)
-                continue;
-
-            uint32_t to_process = next_frame - offs;
-
-            if ((to_process > 0)
-               && (processed < sample_count)
-               && (to_process <= (sample_count - processed)))
-            {
-                int mastered = 0;
-                offs = next_frame;
-                while (to_process - mastered > 0)
-                {
-                    float bpmInc = (float)(processed + mastered - beatsAt) * beats.bpm / (synth->samplerate_f * 60.f);
-                    synth->setBeatValues(beats.songBeat + bpmInc, beats.monotonicBeat + bpmInc, beats.bpm);
-                    int mastered_chunk = _synth->MasterAudio(tmpLeft, tmpRight, to_process - mastered);
-                    for (uint32_t i = 0; i < NUM_MIDI_PARTS + 1; ++i)
-                    {
-                        tmpLeft [i] += mastered_chunk;
-                        tmpRight [i] += mastered_chunk;
-                    }
-
-                    mastered += mastered_chunk;
-                }
-                processed += to_process;
-            }
             //process this midi event
             const uint8_t *msg = (const uint8_t*)(event + 1);
-            if (_bFreeWheel != NULL)
+            if (param_freeWheel)
                 processMidiMessage(msg);
         }
         else if (event->body.type == _atom_blank || event->body.type == _atom_object)
@@ -184,21 +193,32 @@ void YoshimiLV2Plugin::process(uint32_t sample_count)
             LV2_Atom *bar = NULL;
             LV2_Atom *barBeat = NULL;
             LV2_Atom *bpm = NULL;
+            LV2_Atom *beatUnit = NULL;
             lv2_atom_object_get(obj,
                                 _atom_bpb, &bpb,
                                 _atom_bar, &bar,
                                 _atom_bar_beat, &barBeat,
                                 _atom_bpm, &bpm,
+                                _atom_beatUnit, &beatUnit,
                                 NULL);
 
             if (bpm && bpm->type == _atom_float)
             {
                 beats.bpm = ((LV2_Atom_Float *)bpm)->body;
                 bpmProvided = true;
+                if (beatUnit && beatUnit->type == _atom_int)
+                {
+                    // In DAWs, Beats Per Minute really mean Quarter Beats Per
+                    // Minute. Therefore we need to divide by four first, to
+                    // get a whole beat, and then multiply that according to
+                    // the time signature denominator. See this link for some
+                    // background: https://music.stackexchange.com/a/109743
+                    beats.bpm = beats.bpm / 4 * ((LV2_Atom_Int *)beatUnit)->body;
+                }
             }
 
             uint32_t frame = event->time.frames;
-            float bpmInc = (float)(frame - processed) * beats.bpm / (synth->samplerate_f * 60.f);
+            float bpmInc = (float)(frame - processed) * beats.bpm / (synth.samplerate_f * 60.f);
 
             if (bpb && bpb->type == _atom_float
                 && bar && bar->type == _atom_long
@@ -219,40 +239,33 @@ void YoshimiLV2Plugin::process(uint32_t sample_count)
         }
     }
 
-    if (processed < sample_count)
+    while (processed < sample_count)
     {
-        uint32_t to_process = sample_count - processed;
-        int mastered = 0;
-        offs = next_frame;
-        while (to_process - mastered > 0)
+        float bpmInc = (float)(processed - beatsAt) * beats.bpm / (synth.samplerate_f * 60.f);
+        synth.setBeatValues(beats.songBeat + bpmInc, beats.monotonicBeat + bpmInc, beats.bpm);
+        int mastered_chunk = synth.MasterAudio(tmpLeft, tmpRight, sample_count - processed);
+        for (uint32_t i = 0; i < NUM_MIDI_PARTS + 1; ++i)
         {
-            float bpmInc = (float)(processed + mastered - beatsAt) * beats.bpm / (synth->samplerate_f * 60.f);
-            synth->setBeatValues(beats.songBeat + bpmInc, beats.monotonicBeat + bpmInc, beats.bpm);
-            int mastered_chunk = _synth->MasterAudio(tmpLeft, tmpRight, to_process - mastered);
-            for (uint32_t i = 0; i < NUM_MIDI_PARTS + 1; ++i)
-            {
-                tmpLeft [i] += mastered_chunk;
-                tmpRight [i] += mastered_chunk;
-            }
-            mastered += mastered_chunk;
+            tmpLeft [i] += mastered_chunk;
+            tmpRight [i] += mastered_chunk;
         }
-        processed += to_process;
-
+        processed += mastered_chunk;
     }
 
-    float bpmInc = (float)(sample_count - beatsAt) * beats.bpm / (synth->samplerate_f * 60.f);
+    float bpmInc = (float)(sample_count - beatsAt) * beats.bpm / (synth.samplerate_f * 60.f);
     beats.songBeat += bpmInc;
     beats.monotonicBeat += bpmInc;
-    if (!bpmProvided)
-        beats.bpm = synth->PbpmFallback;
+    if (!bpmProvided && lastFallbackBpm != synth.PbpmFallback)
+        beats.bpm = synth.PbpmFallback;
+    lastFallbackBpm = synth.PbpmFallback;
     beatTracker->setBeatValues(beats);
 
     LV2_Atom_Sequence *aSeq = static_cast<LV2_Atom_Sequence *>(_notifyDataPortOut);
     size_t neededAtomSize = sizeof(LV2_Atom_Event) + sizeof(LV2_Atom_Object_Body);
     size_t paddedSize = (neededAtomSize + 7U) & (~7U);
-    if (synth->getNeedsSaving() && _notifyDataPortOut && aSeq->atom.size >= paddedSize) //notify host about plugin's changes
+    if (synth.getNeedsSaving() && _notifyDataPortOut && aSeq->atom.size >= paddedSize) //notify host about plugin's changes
     {
-        synth->setNeedsSaving(false);
+        synth.setNeedsSaving(false);
         aSeq->atom.type = _atom_type_sequence;
         aSeq->atom.size = sizeof(LV2_Atom_Sequence_Body);
         aSeq->body.unit = 0;
@@ -277,28 +290,34 @@ void YoshimiLV2Plugin::process(uint32_t sample_count)
 
 void YoshimiLV2Plugin::processMidiMessage(const uint8_t * msg)
 {
-    bool in_place = _bFreeWheel ? ((*_bFreeWheel == 0) ? false : true) : false;
-    setMidi(msg[0], msg[1], msg[2], in_place);
+    bool in_place = isFreeWheel();
+    handleMidi(msg[0], msg[1], msg[2], in_place);
 }
 
 
-YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const char *bundlePath, const LV2_Feature *const *features, const LV2_Descriptor *desc):
-    MusicIO(synth, new SinglethreadedBeatTracker),
-    _synth(synth),
-    _sampleRate(static_cast<uint32_t>(sampleRate)),
-    _bufferSize(0),
-    _bundlePath(bundlePath),
-    _midiDataPort(NULL),
-    _notifyDataPortOut(NULL),
-    _midi_event_id(0),
-    _bufferPos(0),
-    _offsetPos(0),
-    _bFreeWheel(NULL),
-    _lv2_desc(desc)
+
+YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine& _synth
+                                  ,double sampleRate
+                                  ,const char* bundlePath
+                                  ,LV2_Feature const *const *features
+                                  ,LV2_Descriptor const& lv2Desc
+                                  )
+    : MusicIO(_synth, make_unique<SinglethreadedBeatTracker>())
+    , _sampleRate{static_cast<uint32_t>(sampleRate)}
+    , _bufferSize{0}
+    , _bundlePath{bundlePath}
+    , _midiDataPort{nullptr}
+    , _notifyDataPortOut{nullptr}
+    , _midi_event_id{0}
+    , _bufferPos{0}
+    , _offsetPos{0}
+    , param_freeWheel{nullptr}
+    , flatbankprgs{}
+    , lastFallbackBpm{-1}
+    , isReady{false}
 {
     _uridMap.handle = NULL;
     _uridMap.map = NULL;
-    flatbankprgs.clear();
     const LV2_Feature *f = NULL;
     const Yoshimi_LV2_Options_Option *options = NULL;
     while ((f = *features) != NULL)
@@ -315,7 +334,7 @@ YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const
     }
 
     uint32_t nomBufSize = 0;
-    if (_uridMap.map != NULL && options != NULL)
+    if (_uridMap.map and options)
     {
         _midi_event_id = _uridMap.map(_uridMap.handle, LV2_MIDI__MidiEvent);
         _yoshimi_state_id = _uridMap.map(_uridMap.handle, YOSHIMI_STATE_URI);
@@ -323,7 +342,7 @@ YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const
         LV2_URID maxBufSz = _uridMap.map(_uridMap.handle, YOSHIMI_LV2_BUF_SIZE__maxBlockLength);
         LV2_URID minBufSz = _uridMap.map(_uridMap.handle, YOSHIMI_LV2_BUF_SIZE__minBlockLength);
         LV2_URID nomBufSz = _uridMap.map(_uridMap.handle, YOSHIMI_LV2_BUF_SIZE__nominalBlockLength);
-        LV2_URID atomInt = _uridMap.map(_uridMap.handle, LV2_ATOM__Int);
+        _atom_int = _uridMap.map(_uridMap.handle, LV2_ATOM__Int);
         _atom_long = _uridMap.map(_uridMap.handle, LV2_ATOM__Long);
         _atom_float = _uridMap.map(_uridMap.handle, LV2_ATOM__Float);
         _atom_type_chunk = _uridMap.map(_uridMap.handle, LV2_ATOM__Chunk);
@@ -337,141 +356,134 @@ YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const
         _atom_bar = _uridMap.map(_uridMap.handle, LV2_TIME__bar);
         _atom_bar_beat = _uridMap.map(_uridMap.handle, LV2_TIME__barBeat);
         _atom_bpm = _uridMap.map(_uridMap.handle, LV2_TIME__beatsPerMinute);
+        _atom_beatUnit = _uridMap.map(_uridMap.handle, LV2_TIME__beatUnit);
         while (options->size > 0 && options->value != NULL)
         {
             if (options->context == LV2_OPTIONS_INSTANCE)
             {
-                if ((options->key == minBufSz || options->key == maxBufSz) && options->type == atomInt)
+                if ((options->key == minBufSz || options->key == maxBufSz) && options->type == _atom_int)
                 {
                     uint32_t bufSz = *static_cast<const uint32_t *>(options->value);
                     if (_bufferSize < bufSz)
                         _bufferSize = bufSz;
                 }
-                if (options->key == nomBufSz && options->type == atomInt)
+                if (options->key == nomBufSz && options->type == _atom_int)
                     nomBufSize = *static_cast<const uint32_t *>(options->value);
             }
             ++options;
         }
     }
 
-    //_synth->getRuntime().Log("Buffer size " + to_string(nomBufSize));
+    //runtime().Log("Buffer size " + to_string(nomBufSize));
     if (nomBufSize > 0)
         _bufferSize = nomBufSize;
     else if (_bufferSize == 0)
         _bufferSize = MAX_BUFFER_SIZE;
 
-    synth->setBPMAccurate(true);
+    // Configuration for LV2 mode...
+    runtime().isLV2 = true;
+    runtime().isMultiFeed = isMultiFeed(lv2Desc);
+    synth.setBPMAccurate(true);
 }
 
 
-YoshimiLV2Plugin::~YoshimiLV2Plugin()
-{
-    if (_synth != NULL)
-    {
-        if (!flatbankprgs.empty())
-        {
-            getProgram(flatbankprgs.size() + 1);
-        }
-        _synth->getRuntime().runSynth = false;
-        delete _synth;
-        _synth = NULL;
-    }
 
-    delete beatTracker;
+/** create a new distinct Yoshimi plugin instance; `activate()` will be called prior to `run()`. */
+LV2_Handle YoshimiLV2Plugin::instantiate(LV2_Descriptor const* desc, double sample_rate, const char *bundle_path, LV2_Feature const *const *features)
+{
+    YoshimiLV2Plugin* instance;
+    auto instantiatePlugin = [&](SynthEngine& synth) -> MusicIO*
+                                {
+                                    instance = new YoshimiLV2Plugin(synth, sample_rate, bundle_path, features, *desc);
+                                    return instance;  // note: will be stored/managed in MusicClient
+                                };
+
+    if (Config::instances().startPluginInstance(instantiatePlugin))
+    {
+        assert(instance);
+        instance->isReady.store(true, std::memory_order_release); // after this point, GUI-plugin may attach
+        return static_cast<LV2_Handle>(instance);
+    }
+    else
+        return nullptr;
+}
+
+/** Initialise the plugin instance and activate it for use. */
+void YoshimiLV2Plugin::activate(LV2_Handle h)
+{
+    self(h).runtime().Log("Yoshimi LV2 plugin activated");
+}
+
+void YoshimiLV2Plugin::run(LV2_Handle h, uint32_t sample_count)
+{
+    self(h).process(sample_count);
+}
+
+void YoshimiLV2Plugin::deactivate(LV2_Handle h)
+{
+    self(h).runtime().Log("Yoshimi LV2 plugin deactivated");
+}
+
+/** called by LV2 host to destroy a plugin instance */
+void YoshimiLV2Plugin::cleanup(LV2_Handle h)
+{
+    auto synthID = self(h).synth.getUniqueId();
+    Config::instances().terminatePluginInstance(synthID);
 }
 
 
-bool YoshimiLV2Plugin::init()
+
+bool YoshimiLV2Plugin::openAudio()
 {
-    if (_uridMap.map == NULL || _sampleRate == 0 || _bufferSize == 0 || _midi_event_id == 0 || _yoshimi_state_id == 0 || _atom_string_id == 0)
-        return false;
-    if (!prepBuffers())
-        return false;
+    bool validSettings = not (_uridMap.map == NULL
+                             or _sampleRate == 0
+                             or _bufferSize == 0
+                             or _midi_event_id == 0
+                             or _yoshimi_state_id == 0
+                             or _atom_string_id == 0);
+    return validSettings
+       and prepBuffers();
+}
 
-    if (!_synth->Init(_sampleRate, _bufferSize))
-    {
-        synth->getRuntime().LogError("Can't init synth engine");
-	return false;
-    }
-    if (_synth->getUniqueId() == 0)
-    {
-        firstSynth = _synth;
-        //firstSynth->getRuntime().Log("Started first");
-    }
-
-    _synth->getRuntime().showGui = false;
+bool YoshimiLV2Plugin::openMidi()
+{
+    return true; /*nothing to do*/
+}
 
+bool YoshimiLV2Plugin::Start()
+{
+    // by default do not launch UI; rather create it later, on-demand
+    runtime().showGui = false;
     memset(lv2Left, 0, sizeof(float *) * (NUM_MIDI_PARTS + 1));
     memset(lv2Right, 0, sizeof(float *) * (NUM_MIDI_PARTS + 1));
 
-    _synth->getRuntime().runSynth = true;
-
-    synth->getRuntime().Log("Starting in LV2 plugin mode");
     return true;
 }
 
 
-LV2_Handle	YoshimiLV2Plugin::instantiate (const LV2_Descriptor *desc, double sample_rate, const char *bundle_path, const LV2_Feature *const *features)
-{
-    std::list<string> dummy;
-    LV2PluginType lv2Type;
-    if (std::string(desc->URI) == std::string(yoshimi_lv2_multi_desc.URI))
-        lv2Type = LV2PluginTypeMulti;
-    else
-        lv2Type = LV2PluginTypeSingle;
-    SynthEngine *synth = new SynthEngine(dummy, lv2Type);
-    if (!synth->getRuntime().isRuntimeSetupCompleted())
-    {
-        delete synth;
-        return NULL;
-    }
-    Fl::lock();
-
-    YoshimiLV2Plugin *inst = new YoshimiLV2Plugin(synth, sample_rate, bundle_path, features, desc);
-    if (inst->init())
-    {
-        /*
-        * Perform further global initialisation.
-        * For stand-alone the equivalent init happens in main(),
-        * after mainCreateNewInstance() returned successfully.
-        */
-        synth->installBanks();
-        synth->loadHistory();
-        return static_cast<LV2_Handle>(inst);
-    }
-    else
-    {
-        synth->getRuntime().LogError("Failed to create Yoshimi LV2 plugin");
-        delete inst;
-        delete synth;
-    }
-    return NULL;
-}
-
-
-void YoshimiLV2Plugin::connect_port(LV2_Handle instance, uint32_t port, void *data_location)
+void YoshimiLV2Plugin::connect_port(LV2_Handle handle, uint32_t port, void* data_location)
 {
     if (port > NUM_MIDI_PARTS + 2)
         return;
-     YoshimiLV2Plugin *inst = static_cast<YoshimiLV2Plugin *>(instance);
+    YoshimiLV2Plugin& plugin = self(handle);
      if (port == 0)//atom midi event port
      {
-         inst->_midiDataPort = static_cast<LV2_Atom_Sequence *>(data_location);
+         plugin._midiDataPort = static_cast<LV2_Atom_Sequence*>(data_location);
          return;
      }
      else if (port == 1) //freewheel control port
      {
-         inst->_bFreeWheel = static_cast<float *>(data_location);
+         plugin.param_freeWheel = static_cast<float*>(data_location);
          return;
      }
-     else if (port == 36 && std::string(inst->_lv2_desc->URI) == std::string(yoshimi_lv2_multi_desc.URI)) //notify out port
+     else if (port == 36 and plugin.runtime().isMultiFeed) //notify out port
      {
-         inst->_notifyDataPortOut = static_cast<LV2_Atom_Sequence *>(data_location);
+         plugin._notifyDataPortOut = static_cast<LV2_Atom_Sequence*>(data_location);
          return;
      }
-     else if (port == 4 && std::string(inst->_lv2_desc->URI) == std::string(yoshimi_lv2_desc.URI)) //notify out port
+     else if (port == 4 and not plugin.runtime().isMultiFeed) //notify out port
      {
-         inst->_notifyDataPortOut = static_cast<LV2_Atom_Sequence *>(data_location);
+         plugin._notifyDataPortOut = static_cast<LV2_Atom_Sequence*>(data_location);
          return;
      }
 
@@ -486,59 +498,24 @@ void YoshimiLV2Plugin::connect_port(LV2_Handle instance, uint32_t port, void *da
 
      int portIndex = static_cast<int>(floorf((float)port/2.0f));
      if (port % 2 == 0) //left channel
-         inst->lv2Left[portIndex] = static_cast<float *>(data_location);
+         plugin.lv2Left[portIndex] = static_cast<float *>(data_location);
      else
-         inst->lv2Right[portIndex] = static_cast<float *>(data_location);
+         plugin.lv2Right[portIndex] = static_cast<float *>(data_location);
 }
 
 
-void YoshimiLV2Plugin::activate(LV2_Handle instance)
-{
-    YoshimiLV2Plugin *inst = static_cast<YoshimiLV2Plugin *>(instance);
-    inst->Start();
-}
-
-
-void YoshimiLV2Plugin::deactivate(LV2_Handle instance)
-{
-    YoshimiLV2Plugin *inst = static_cast<YoshimiLV2Plugin *>(instance);
-    inst->Close();
-}
-
-
-void YoshimiLV2Plugin::run(LV2_Handle instance, uint32_t sample_count)
-{
-    YoshimiLV2Plugin *inst = static_cast<YoshimiLV2Plugin *>(instance);
-    inst->process(sample_count);
-}
-
-void YoshimiLV2Plugin::cleanup(LV2_Handle instance)
-{
-    YoshimiLV2Plugin *inst = static_cast<YoshimiLV2Plugin *>(instance);
-    delete inst;
-}
-
-
-/*
-LV2_Worker_Interface yoshimi_wrk_iface =
-{
-    YoshimiLV2Plugin::lv2wrk_work,
-    YoshimiLV2Plugin::lv2wrk_response,
-    YoshimiLV2Plugin::lv2_wrk_end_run
-};
-*/
 
 LV2_Programs_Interface yoshimi_prg_iface =
 {
-    YoshimiLV2Plugin::static_GetProgram,
-    YoshimiLV2Plugin::static_SelectProgram,
-    YoshimiLV2Plugin::static_SelectProgramNew
+    YoshimiLV2Plugin::callback_getProgram,
+    YoshimiLV2Plugin::callback_selectProgram,
+    YoshimiLV2Plugin::callback_selectProgramNew
 };
 
 
 const void *YoshimiLV2Plugin::extension_data(const char *uri)
 {
-    static const LV2_State_Interface state_iface = { YoshimiLV2Plugin::static_StateSave, YoshimiLV2Plugin::static_StateRestore };
+    static const LV2_State_Interface state_iface = { YoshimiLV2Plugin::callback_stateSave, YoshimiLV2Plugin::callback_stateRestore };
     if (!strcmp(uri, LV2_STATE__interface))
     {
         return static_cast<const void *>(&state_iface);
@@ -565,7 +542,7 @@ LV2_State_Status YoshimiLV2Plugin::stateSave(LV2_State_Store_Function store, LV2
     // suppress warnings - may use later
 
     char *data = NULL;
-    int sz = _synth->getalldata(&data);
+    int sz = synth.getalldata(&data);
 
     store(handle, _yoshimi_state_id, data, sz, _atom_string_id, LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE);
     free(data);
@@ -587,263 +564,208 @@ LV2_State_Status YoshimiLV2Plugin::stateRestore(LV2_State_Retrieve_Function retr
     const char *data = (const char *)retrieve(handle, _yoshimi_state_id, &sz, &type, &new_flags);
 
     if (sz > 0)
-        _synth->putalldata(data, sz);
+        synth.putalldata(data, sz);
     return LV2_STATE_SUCCESS;
 }
 
 
-const LV2_Program_Descriptor *YoshimiLV2Plugin::getProgram(uint32_t index)
+LV2_Program_Descriptor const * YoshimiLV2Plugin::getProgram(uint32_t index)
 {
     if (flatbankprgs.empty())
     {
-        Bank &bankObj = synth->getBankRef();
-        const BankEntryMap &banks = bankObj.getBanks(synth->getRuntime().currentRoot);
-        BankEntryMap::const_iterator itB;
-        InstrumentEntryMap::const_iterator itI;
-        for (itB = banks.begin(); itB != banks.end(); ++itB)
-        {
-            string bankName = itB->second.dirname;
-            if (!bankName.empty())
-            {
-                for (itI = itB->second.instruments.begin(); itI != itB->second.instruments.end(); ++itI)
-                {
-                    if (!itI->second.name.empty())
+        BankEntryMap const& banks{synth.bank.getBanks(runtime().currentRoot)};
+        for (auto& [bankID,bank] : banks)
+            if (not bank.dirname.empty())
+                for (auto& [instrumentID,instrument] : bank.instruments)
+                    if (not instrument.name.empty())
                     {
-                        LV2_Program_Descriptor desc;
-                        desc.bank = itB->first;
-                        desc.program = itI->first;
-                        desc.name = strdup((bankName + " -> " + itI->second.name).c_str());
-                        flatbankprgs.push_back(desc);
+                        LV2Bank entry;
+                        entry.bank    = bankID;
+                        entry.program = instrumentID;
+                        entry.display = bank.dirname + " -> " + instrument.name;
+                        entry.name    = entry.display.c_str();
+                        flatbankprgs.push_back(entry);
                     }
-                }
-            }
-        }
     }
-
-    if (index >= flatbankprgs.size())
-    {
-        for (size_t i = 0; i < flatbankprgs.size(); ++i)
-        {
-            if (flatbankprgs [i].name != NULL)
-            {
-                free(const_cast<char *>(flatbankprgs [i].name));
-            }
-        }
-        flatbankprgs.clear();
-        return NULL;
-    }
-    return &flatbankprgs [index];
+    return index < flatbankprgs.size()? &flatbankprgs [index]
+                                      : nullptr;
 }
 
 
 void YoshimiLV2Plugin::selectProgramNew(unsigned char channel, uint32_t bank, uint32_t program)
 {
-    bool isFreeWheel = false;
-    if (_bFreeWheel && *_bFreeWheel == 1)
-        isFreeWheel = true;
-    if (_synth->getRuntime().midi_bank_C != 128)
+    if (runtime().midi_bank_C != 128)
     {
-        synth->mididecode.setMidiBankOrRootDir((short)bank, isFreeWheel);
+        synth.mididecode.setMidiBankOrRootDir((short)bank, isFreeWheel());
     }
-    synth->mididecode.setMidiProgram(channel, program, isFreeWheel);
+    synth.mididecode.setMidiProgram(channel, program, isFreeWheel());
 }
 
 
-LV2_State_Status YoshimiLV2Plugin::static_StateSave(LV2_Handle instance, LV2_State_Store_Function store, LV2_State_Handle handle, uint32_t flags, const LV2_Feature * const *features)
+LV2_State_Status YoshimiLV2Plugin::callback_stateSave(LV2_Handle h, LV2_State_Store_Function store, LV2_State_Handle state, uint32_t flags, const LV2_Feature * const *features)
 {
-    return static_cast<YoshimiLV2Plugin *>(instance)->stateSave(store, handle, flags, features);
+    return self(h).stateSave(store, state, flags, features);
 }
 
 
-LV2_State_Status YoshimiLV2Plugin::static_StateRestore(LV2_Handle instance, LV2_State_Retrieve_Function retrieve, LV2_State_Handle handle, uint32_t flags, const LV2_Feature * const *features)
+LV2_State_Status YoshimiLV2Plugin::callback_stateRestore(LV2_Handle h, LV2_State_Retrieve_Function retrieve, LV2_State_Handle state, uint32_t flags, const LV2_Feature * const *features)
 {
-    return static_cast<YoshimiLV2Plugin *>(instance)->stateRestore(retrieve, handle, flags, features);
+    return self(h).stateRestore(retrieve, state, flags, features);
 }
 
 
-const LV2_Program_Descriptor *YoshimiLV2Plugin::static_GetProgram(LV2_Handle handle, uint32_t index)
+const LV2_Program_Descriptor *YoshimiLV2Plugin::callback_getProgram(LV2_Handle h, uint32_t index)
 {
-    return static_cast<YoshimiLV2Plugin *>(handle)->getProgram(index);
+    return self(h).getProgram(index);
 }
 
 
-void YoshimiLV2Plugin::static_SelectProgramNew(LV2_Handle handle, unsigned char channel, uint32_t bank, uint32_t program)
+void YoshimiLV2Plugin::callback_selectProgramNew(LV2_Handle h, unsigned char channel, uint32_t bank, uint32_t program)
 {
-    return static_cast<YoshimiLV2Plugin *>(handle)->selectProgramNew(channel, bank, program);
+    return self(h).selectProgramNew(channel, bank, program);
 }
 
 
-/*
-LV2_Worker_Status YoshimiLV2Plugin::lv2wrk_work(LV2_Handle instance, LV2_Worker_Respond_Function respond, LV2_Worker_Respond_Handle handle, uint32_t size, const void *data)
+
+
+YoshimiLV2PluginUI::YoshimiLV2PluginUI(const char *, LV2UI_Write_Function, LV2UI_Controller controller,
+                                       LV2UI_Widget* widget, LV2_Feature const *const *features)
+    : corePlugin(nullptr)
+    , plugin_human_id{"Yoshimi lv2 plugin"}
+    , notify_on_GUI_close{}
 {
+    // Configure callbacks for running the UI
+    LV2_External_UI_Widget::run  = YoshimiLV2PluginUI::callback_Run;
+    LV2_External_UI_Widget::show = YoshimiLV2PluginUI::callback_Show;
+    LV2_External_UI_Widget::hide = YoshimiLV2PluginUI::callback_Hide;
 
-}
-
-
-LV2_Worker_Status YoshimiLV2Plugin::lv2wrk_response(LV2_Handle instance, uint32_t size, const void *body)
-{
-
-}
-
-
-LV2_Worker_Status YoshimiLV2Plugin::lv2_wrk_end_run(LV2_Handle instance)
-{
-
-}
-
-*/
-
-
-YoshimiLV2PluginUI::YoshimiLV2PluginUI(const char *, LV2UI_Write_Function write_function, LV2UI_Controller controller, LV2UI_Widget *widget, const LV2_Feature * const *features)
-    :_plugin(NULL),
-     _masterUI(NULL),
-     _controller(controller),
-     _write_function(write_function)
-{
-    uiHost.plugin_human_id = NULL;
-    uiHost.ui_closed = NULL;
-    const LV2_Feature *f = NULL;
-    externalUI.uiWIdget.run = YoshimiLV2PluginUI::static_Run;
-    externalUI.uiWIdget.show = YoshimiLV2PluginUI::static_Show;
-    externalUI.uiWIdget.hide = YoshimiLV2PluginUI::static_Hide;
-    externalUI.uiInst = this;
-    while ((f = *features) != NULL)
+    while (*features)
     {
+        LV2_Feature const* f = *features;
         if (strcmp(f->URI, LV2_INSTANCE_ACCESS_URI) == 0)
         {
-            _plugin = static_cast<YoshimiLV2Plugin *>(f->data);
+            corePlugin = static_cast<YoshimiLV2Plugin *>(f->data);
         }
         else if (strcmp(f->URI, LV2_EXTERNAL_UI__Host) == 0)
         {
-            uiHost.plugin_human_id = strdup(static_cast<LV2_External_UI_Host *>(f->data)->plugin_human_id);
-            uiHost.ui_closed = static_cast<LV2_External_UI_Host *>(f->data)->ui_closed;
+            LV2_External_UI_Host& hostSpec = * static_cast<LV2_External_UI_Host *>(f->data);
+            plugin_human_id = hostSpec.plugin_human_id;
+            auto callback = hostSpec.ui_closed;
+            notify_on_GUI_close = [callback,controller]{ callback(controller); };
         }
         ++features;
     }
-    if (uiHost.plugin_human_id == NULL)
-    {
-        uiHost.plugin_human_id = strdup("Yoshimi lv2 plugin");
-    }
-    *widget = &externalUI;
+    // this object also serves as »widget« for the event callbacks
+    *widget = static_cast<LV2UI_Widget>(this);
 }
 
 YoshimiLV2PluginUI::~YoshimiLV2PluginUI()
 {
-    if (uiHost.plugin_human_id != NULL)
-    {
-        free(const_cast<char *>(uiHost.plugin_human_id));
-        uiHost.plugin_human_id = NULL;
-    }
-    _plugin->_synth->closeGui();
-    Fl::check(); // TODO do we need this?
+    engine().shutdownGui();
+    Fl::check(); // necessary to ensure screen redraw after all windows are hidden
 }
 
 
 bool YoshimiLV2PluginUI::init()
 {
-    if (_plugin == NULL || uiHost.ui_closed == NULL)
+    if (not (corePlugin and notify_on_GUI_close))
         return false;
 
-    _plugin->_synth->setGuiClosedCallback(YoshimiLV2PluginUI::static_guiClosed, this);
+    // LV2 hosts may load plugins concurrently, which in some corner cases
+    // causes a race between SynthEngine initialisation and bootstrap of the GUI
+    while (not corePlugin->isReady.load(std::memory_order_acquire))
+        std::this_thread::yield();
 
+    engine().installGuiClosedCallback([this]
+                                        {// invoked by SynthEngine when FLTK GUI is closed explicitly...
+                                            engine().shutdownGui();
+                                            notify_on_GUI_close();
+                                        });
     return true;
 }
 
 
-LV2UI_Handle YoshimiLV2PluginUI::instantiate(const LV2UI_Descriptor *descriptor, const char *plugin_uri, const char *bundle_path, LV2UI_Write_Function write_function, LV2UI_Controller controller, LV2UI_Widget *widget, const LV2_Feature * const *features)
+/** activated by LV2 host when preparing to launch the GUI: create a UI plugin associated with a core plugin instance */
+LV2UI_Handle YoshimiLV2PluginUI::instantiate(LV2UI_Descriptor const*, const char* /*plugin_uri*/, const char* bundle_path,
+                                             LV2UI_Write_Function write_function, LV2UI_Controller controller,
+                                             LV2UI_Widget* widget, const LV2_Feature * const *features)
 {
-    const LV2UI_Descriptor *desc = descriptor;
-    descriptor = desc;
-    const char *plug = plugin_uri;
-    plugin_uri = plug;
-    // lines above suppress warnings - may use later
-
-    YoshimiLV2PluginUI *uiinst = new YoshimiLV2PluginUI(bundle_path, write_function, controller, widget, features);
+    YoshimiLV2PluginUI* uiinst = new YoshimiLV2PluginUI(bundle_path, write_function, controller, widget, features);
     if (uiinst->init())
     {
-        return static_cast<LV2_External_UI_Widget *>(uiinst);
+        return static_cast<LV2UI_Handle>(uiinst);
     }
     else
         delete uiinst;
     return NULL;
 }
 
-
+/** called by LV2 host to discard the plugin GUI;
+ *  alternatively the host may choose just to hide the UI */
 void YoshimiLV2PluginUI::cleanup(LV2UI_Handle ui)
 {
     YoshimiLV2PluginUI *uiinst = static_cast<YoshimiLV2PluginUI *>(ui);
     delete uiinst;
 }
 
-
-void YoshimiLV2PluginUI::static_guiClosed(void *arg)
-{
-    static_cast<YoshimiLV2PluginUI *>(arg)->_masterUI = NULL;
-    static_cast<YoshimiLV2PluginUI *>(arg)->_plugin->_synth->closeGui();
-}
-
-
+/** recurring GUI event handling cycle*/
 void YoshimiLV2PluginUI::run()
 {
-    if (_masterUI != NULL)
+    if (isGuiActive())
     {
-        _masterUI->checkBuffer();
+        masterUI().checkBuffer();
         Fl::check();
     }
     else
-    {
-        if (uiHost.ui_closed != NULL)
-            uiHost.ui_closed(_controller);
-    }
+        notify_on_GUI_close();
 }
 
 
 void YoshimiLV2PluginUI::show()
 {
-    _plugin->_synth->getRuntime().showGui = true;
-    bool bInit = false;
-    if (_masterUI == NULL)
-        bInit = true;
-    _masterUI = _plugin->_synth->getGuiMaster();
-    if (_masterUI == NULL)
+    if (not isGuiActive())
     {
-        _plugin->_synth->getRuntime().Log("Failed to instantiate gui");
-        return;
+        initFltkLock();
+        Config::instances().launchGui_forPlugin(engine().getUniqueId(), plugin_human_id);
     }
-    if (bInit)
-        _masterUI->Init(uiHost.plugin_human_id);
+    else
+        masterUI().masterwindow->show();
 }
 
 
 void YoshimiLV2PluginUI::hide()
 {
-    if (_masterUI)
-        _masterUI->masterwindow->hide();
+    if (isGuiActive())
+        masterUI().masterwindow->hide();
 }
 
 
-void YoshimiLV2PluginUI::static_Run(_LV2_External_UI_Widget *_this_)
+/**
+ * This function was introduced as an attempt to be defensive and handle FLTK locking properly.
+ * We use the LV2 extension "http://yoshimi.sourceforge.net/lv2_plugin#ExternalUI"
+ * and I could not find any documentation to rule out that some host may invoke the
+ * LV2_External_UI_Widget run() function concurrently for two distinct GUI instances.
+ * On the other hand, FLTK by default runs single threaded (as most UI toolkits to)
+ * and has clearly stated rules how to deal with concurrency.
+ * See https://www.fltk.org/doc-1.3/advanced.html
+ * This function ensures thus that the Fl::lock() is set _once and initially_,
+ * before creating the first window.
+ */
+void YoshimiLV2PluginUI::initFltkLock()
 {
-    reinterpret_cast<_externalUI *>(_this_)->uiInst->run();
-
-}
-
-
-void YoshimiLV2PluginUI::static_Show(_LV2_External_UI_Widget *_this_)
-{
-    reinterpret_cast<_externalUI *>(_this_)->uiInst->show();
-}
-
-
-void YoshimiLV2PluginUI::static_Hide(_LV2_External_UI_Widget *_this_)
-{
-    reinterpret_cast<_externalUI *>(_this_)->uiInst->hide();
-
+    static bool firstTime{true};
+    if (firstTime)
+    {
+        Fl::lock();
+        firstTime = false;
+    }
 }
 
 
 
-extern "C" const LV2_Descriptor *lv2_descriptor(uint32_t index)
+
+/** Entry point for the Host to discover and load the core plugin */
+extern "C" const LV2_Descriptor* lv2_descriptor(uint32_t index)
 {
     switch (index)
     {
@@ -868,6 +790,7 @@ LV2UI_Descriptor yoshimi_lv2ui_desc =
 };
 
 
+/** Entry point for the Host to discover and load the associated GUI plugin */
 extern "C" const LV2UI_Descriptor* lv2ui_descriptor(uint32_t index)
 {
     switch (index)
@@ -880,15 +803,3 @@ extern "C" const LV2UI_Descriptor* lv2ui_descriptor(uint32_t index)
     return NULL;
 
 }
-
-
-int mainCreateNewInstance(unsigned int) //stub
-{
-    return 0;
-}
-
-
-void mainRegisterAudioPort(SynthEngine *, int ) //stub
-{
-
-}
