@@ -478,7 +478,12 @@ bool Config::initFromPersistentConfig()
         int currentVersion = lastXMLmajor * 10 + lastXMLminor;
         int storedVersion = MIN_CONFIG_MAJOR * 10 + MIN_CONFIG_MINOR;
         if (currentVersion < storedVersion)
+        {
             oldConfig = true;
+            saveInstanceConfig();
+            // Always resave to fix.
+            // User may wish to accept this unchanged.
+        }
         else
             oldConfig = false;
     }
@@ -513,7 +518,10 @@ bool Config::initFromPersistentConfig()
         if (pos != string::npos)
             currentV = currentV.substr(0,pos);
         if (currentV == guideVersion && isRegularFile(manualFile))
+        {
             man_ok = true;
+            //std::cout << "Manual already seen" << std::endl;
+        }
 
         if (!man_ok)
         {
@@ -1450,15 +1458,21 @@ std::string Config::findHtmlManual()
 {
     string namelist = "";
     string tempnames = "";
-    if(file::cmd2string("find /usr/share/ -xdev -type f -name 'yoshimi_user_guide_version' 2>/dev/null", tempnames))
+    if(file::cmd2string("find /usr/share/doc/ -xdev -type f -name 'yoshimi_user_guide_version' 2>/dev/null", tempnames))
+    {
         namelist = tempnames;
+        tempnames = "";
+    }
 
-    if(file::cmd2string("find /usr/local/share/ -xdev -type f -name 'yoshimi_user_guide_version' 2>/dev/null", tempnames))
+    if(file::cmd2string("find /usr/local/share/doc/ -xdev -type f -name 'yoshimi_user_guide_version' 2>/dev/null", tempnames))
+    {
         namelist += tempnames;
+        tempnames = "";
+    }
 
-    if(file::cmd2string("find /home/ -xdev -type f -name 'yoshimi_user_guide_version' 2>/dev/null", tempnames))
+    if(file::cmd2string("find $HOME/.local/share/doc/yoshimi/ -xdev -type f -name 'yoshimi_user_guide_version' 2>/dev/null", tempnames))
         namelist += tempnames;
-
+    //std::cout << "Manual lists\n" << namelist << std::endl;
     size_t next = 0;
     string lastversion = "";
     string found = "";
@@ -1471,7 +1485,7 @@ std::string Config::findHtmlManual()
         {
             name = namelist.substr(0, next);
             current = loadText(name);
-            if (current > lastversion)
+            if (current >= lastversion)
             {
                 lastversion = current;
                 found = name;
